package com.example.onlinejudge.model.VO;

import cn.hutool.json.JSONUtil;
import com.example.onlinejudge.model.dto.question.JudgeConfig;
import com.example.onlinejudge.model.entity.Question;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.beans.BeanUtils;

import java.util.Date;
import java.util.List;

@ApiModel("题目封装类")
@Data
public class QuestionVO {

    @ApiModelProperty("题目ID")
    private Long id;

    @ApiModelProperty("题目标题")
    private String title;

    @ApiModelProperty("题目内容")
    private String content;

    @ApiModelProperty("题目标签")
    private List<String> tags;

    @ApiModelProperty("题目提交数")
    private Integer submitNum;

    @ApiModelProperty("题目通过数")
    private Integer acceptedNum;

    @ApiModelProperty("判题配置")
    private JudgeConfig judgeConfig;

    @ApiModelProperty("创建用户 id")
    private Long userId;

    @ApiModelProperty("创建时间")
    private Date createTime;

    @ApiModelProperty("更新时间")
    private Date updateTime;

    @ApiModelProperty("创建用户信息")
    private UserVO userVO;

    /**
     * 包装类转对象
     *
     * @param questionVO
     * @return
     */
    public static Question voToObj(QuestionVO questionVO) {
        if (questionVO == null) {
            return null;
        }
        Question question = new Question();
        BeanUtils.copyProperties(questionVO, question);
        List<String> tagList = questionVO.getTags();
        if (tagList != null) {
            question.setTags(JSONUtil.toJsonStr(tagList));
        }
        JudgeConfig voJudgeConfig = questionVO.getJudgeConfig();
        if (voJudgeConfig != null) {
            question.setJudgeConfig(JSONUtil.toJsonStr(voJudgeConfig));
        }
        return question;
    }

    /**
     * 对象转包装类
     *
     * @param question
     * @return
     */
    public static QuestionVO objToVo(Question question) {
        if (question == null) {
            return null;
        }
        QuestionVO questionVO = new QuestionVO();
        BeanUtils.copyProperties(question, questionVO);
        List<String> tagList = JSONUtil.toList(question.getTags(), String.class);
        questionVO.setTags(tagList);
        String judgeConfigStr = question.getJudgeConfig();
        questionVO.setJudgeConfig(JSONUtil.toBean(judgeConfigStr, JudgeConfig.class));
        return questionVO;
    }

    private static final long serialVersionUID = 1L;
}
