package com.example.onlinejudge.model.entity.result;

import lombok.Data;

import java.io.Serializable;

@Data
public class Result<T> implements Serializable {
    private Integer code;
    private String message;
    private T data;
    public Result() {

    }
    public Result(int code, String message) {
        this.code = code;
        this.message = message;
    }

    public Result(int code, T data){
        this(code, "", data);
    }
    public Result(int code, String message,T data) {
        this.code = code;
        this.message = message;
        this.data = data;
    }

    private void setResultCode(ResultCode resultCode) {
        this.code = resultCode.getCode();
        this.message = resultCode.getMessage();
    }

    /**
     * 成功
     */
    public static <T> Result<T> success() {
        Result<T> result = new Result<T>();
        result.setCode(ResultCode.SUCCESS.getCode());
        result.setMessage(ResultCode.SUCCESS.getMessage());
        return result;
    }

    /**
     * 成功
     */
    public static <T> Result<T> success(T data) {
        Result<T> result = new Result<T>();
        result.setCode(ResultCode.SUCCESS.getCode());
        result.setMessage(ResultCode.SUCCESS.getMessage());
        result.setData(data);
        return result;
    }

    /**
     * 失败
     */
    public static <T> Result<T> error(int code, String message) {
        return new Result(code, message);
    }

    /**
     * 失败
     */
    public static <T> Result<T> error(ResultCode resultCode) {
        return new Result(resultCode.getCode(), resultCode.getMessage());
    }

    /**
     * 判断数据库操作
     */
    public static <T> Result<T> dbResult(int result) {
        if (result > 0) {
            return Result.success(null);
        } else {
            return Result.error(ResultCode.DATABASE_ERROR);
        }
    }
}
